import logging
from colorama import init, Fore, Back

init(autoreset=True)


class ColorFormatter(logging.Formatter):
    # Change this dictionary to suit your coloring needs!
    COLORS = {
        "WARNING": Fore.RED,
        "ERROR": Fore.RED + Back.WHITE,
        "DEBUG": Fore.LIGHTYELLOW_EX,
        "INFO": Fore.BLUE,
        "CRITICAL": Fore.RED + Back.WHITE,
    }

    def format(self, record):
        color = self.COLORS.get(record.levelname, "")
        if color:
            record.name = color + record.name
            record.levelname = color + record.levelname
            record.msg = color + record.msg
        return logging.Formatter.format(self, record)


class ColorLogger(logging.Logger):
    def __init__(self, name):
        logging.Logger.__init__(self, name, logging.DEBUG)
        color_formatter = ColorFormatter("%(name)-10s %(message)s")
        console = logging.StreamHandler()
        console.setFormatter(color_formatter)
        self.addHandler(console)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['whaleman']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'colorama==0.4.5', 'docker>=6.0.0,<7.0.0']

entry_points = \
{'console_scripts': ['whaleman = whaleman:main']}

setup_kwargs = {
    'name': 'whaleman',
    'version': '0.1.8',
    'description': 'docker image versioning',
    'long_description': '## Intro\nwhaleman It is a simple tool to build , push and tag docker images.\n\nusing config .ini file to configure the tool. its annoying sometimes we tagging the image over and over again.\n\nwith this we can simply use config for tagging and versioning for each project/dockerfile\n\n\n## Installation\n\n```bash\npip install whaleman\n```\n\n## Usage\nby default name of the config file is docker.ini\n\nthe content of the config file is\n\n```ini\n[registry]\nname = index.docker.io # name of the registry URL , default is index.docker.io\nlogin = True # if you want to login to the registry , default is False\n# username for the registry \n# default is None or grab from env var with name DOCKER_USERNAME\ndocker_username = myusername \n# password for the registry , use quotes if you have symbol in the password.\n# default is None or grab from env var with name DOCKER_PASSWORD\ndocker_password = "mypassword"  \n\n[image]\nname = soberdev/homepage # this is name of the image, required\ntag = 0.1.8 # this is the version of the image or initial version, required\n```\n\nor if you want to create a config file simply run:\n\n```bash\nwhaleman createconfig\n```\n\nthe createconfig takes arguments, by default it will create a config file with name docker.ini\n\nbut you can create a config file with any name you want with the argument -n\n\n```bash\nwhaleman createconfig -n myconfig.ini\n```\n### DockerHost\nby default whaleman will try grab the value from env variables **DOCKER_HOST** if the environment is empty, then default to **unix://var/run/docker.sock**\n### Run the tool\n\npatch choices are **keep, micro, minor and major**\n\n**keep** will not update the version\n\n```bash\nwhaleman build <patch>\n```\n\nbuild the image and patch the version, after done will auto increment the version in the config file\n\nyou can build and push the image to the registry with the positional argument **push**\n\n```bash\nwhaleman build <patch> push\n```\nchange the registry uri dynamically\n\n```bash\nwhaleman build <patch> push -r your.registry.url\n```\n\nor you can use any config file and custom dockerfilename\n\n```bash\nwhaleman build <patch> push -f myconfig.ini -df mydockerfile.Dockerfile\n```\n\n## For more information\n```bash\nwhaleman -h\n```\n\n```bash\nwhaleman build -h\n```\n\n```bash\nwhaleman createconfig -h\n```\n\n## Logging Level\n\ndefault is INFO, but you can change it with the argument -l\n\n```bash\nwhaleman -l DEBUG build <patch>\n```',
    'author': 'rede akbar wijaya',
    'author_email': 'rede@soberdev.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from .library import DockerRun, DockerRunCreateConfig
import argparse

parser = argparse.ArgumentParser(description="Process Docker images.")
subparser = parser.add_subparsers(dest="command")
subparser.required = True
buildparser = subparser.add_parser("build")
parser.add_argument(
    "--level",
    "-l",
    default="INFO",
    help="log level",
    choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
)
createconfigparser = subparser.add_parser("createconfig")
createconfigparser.add_argument(
    "--filename", "-n", help="config file names", default="docker.ini", type=str
)

buildparser.add_argument(
    "version",
    default="keep",
    const="keep",
    nargs="?",
    choices=["keep", "micro", "minor", "major"],
    help="image patch build version",
)
buildparser.add_argument("push", type=str, nargs="?", help="to push docker images")
buildparser.add_argument("-r", "--registry", help="Config custom registry URL")
buildparser.add_argument(
    "-f", "--filename", help="Config custom file name", default="docker.ini"
)
buildparser.add_argument(
    "-df", "--dockerfile", type=str, help="custom name dockerfile", default="Dockerfile"
)
GETCLASS = dict(
    build=DockerRun,
    createconfig=DockerRunCreateConfig,
)


def main() -> None:
    args = parser.parse_args()
    cls = GETCLASS[args.command](args)
    cls()



from multiprocessing import AuthenticationError
from docker import APIClient
import configparser
import os
import click
from os.path import exists as file_exists
from .logs import ColorLogger
import logging

logging.setLoggerClass(ColorLogger)

logger = logging.getLogger(__name__)


class Versioner:
    def __init__(self, version: str) -> None:
        self.major, self.minor, self.micro = version.split(".")
        self.major = int(self.major)
        self.minor = int(self.minor)
        self.micro = int(self.micro)

    def patch_micro(self):
        self.micro += 1

    def patch_minor(self):
        self.minor += 1
        self.micro = 0

    def patch_major(self):
        self.major += 1
        self.minor = 0
        self.micro = 0

    @property
    def version(self) -> str:
        return "{}.{}.{}".format(self.major, self.minor, self.micro)


class BaseRun:
    client = APIClient(base_url=os.getenv("DOCKER_HOST", "unix://var/run/docker.sock"))
    config = configparser.ConfigParser()

    def __init__(self, args) -> None:
        logger.setLevel(args.level)
        self.args = args
        self.is_ini_extension()

    def is_ini_extension(self):
        if not self.args.filename.endswith(".ini"):
            raise ValueError("file extension must be .ini")


class DockerRun(BaseRun):
    def __init__(self, args) -> None:
        super().__init__(args)
        if not file_exists(self.args.filename):
            raise NotImplementedError(f"{self.args.filename} not found")
        self.config.read(self.args.filename)
        self.is_login = self.config.getboolean("registry", "login")
        self.registry = args.registry or self.config.get("registry", "name")
        self.tag = self.config.get("image", "tag")
        self.version = Versioner(self.tag)
        logger.debug("registry: {}".format(self.registry))
        logger.debug("prev version: {}".format(self.version.version))
        if args.version != "keep":
            getattr(self.version, f"patch_{args.version}")()
        logger.debug("next version: {}".format(self.version.version))
        self.image = self.config.get("image", "name")
        logger.debug("image name: {}".format(self.image))
        self.image_tag = "{}/{}:{}".format(
            self.registry, self.image, self.version.version
        )
        logger.debug("image tag: {}".format(self.image_tag))
        self.image_tag_latest = "{}/{}:{}".format(self.registry, self.image, "latest")
        logger.debug("image tag latest: {}".format(self.image_tag_latest))

    def login(self):
        logger.debug("Start login")
        try:
            self.client.login(
                username=self.config.get(
                    "registry", "docker_username", fallback=None, vars=os.environ
                ),
                password=self.config.get(
                    "registry", "docker_password", fallback=None, vars=os.environ
                ),
                registry=self.registry,
            )
            logger.info("Logged in to {}".format(self.registry))
        except Exception as e:
            logger.error("Failed to login to {}".format(self.registry))
            logger.error(e)
            raise AuthenticationError("Failed to login to {}".format(self.registry))

    def build(self, tag) -> None:
        logger.debug("Start build")
        logger.debug("building image with tag: {}".format(tag))
        result = self.client.build(
            path=".", tag=tag, decode=True, dockerfile=self.args.dockerfile
        )
        for chunk in result:
            if "stream" in chunk:
                for line in chunk["stream"].splitlines():
                    logger.info(line)

    def push(self, repo_tag) -> None:
        if self.is_login:
            self.login()
        result = self.client.push(repo_tag, stream=True, decode=True)
        for line in result:
            logger.info(
                f"{line.get('status', '-')} {line.get('progress', '-')}  {line.get('id', '-')}"
            )

    def __call__(self) -> None:
        self.build(self.image_tag)
        logger.debug("image with tag: {}".format(self.image_tag))
        self.client.tag(self.image_tag, self.image_tag_latest)
        logger.debug("retag image with latest tag: {}".format(self.image_tag_latest))
        if self.args.push:
            self.push(self.image_tag)
            self.push(self.image_tag_latest)
        self.config.set("image", "tag", self.version.version)
        with open(self.args.filename, "w") as configfile:
            self.config.write(configfile)


class DockerRunCreateConfig(BaseRun):
    @staticmethod
    def check_is_valid_version(version: str) -> bool:
        # version format should be like "1.0.0"
        if not version.count(".") == 2:
            raise ValueError("version format should be like 1.0.0")
        list_ver: list = version.split(".")
        try:
            [int(i) for i in list_ver]
        except ValueError:
            raise ValueError("version format only support int")
        return version

    def __call__(self) -> None:
        self.config.add_section("image")
        self.config.add_section("registry")
        image_name = click.prompt(click.style("input image name", fg="green"), type=str)
        tag = click.prompt(
            click.style("input initial tag", fg="green"),
            type=str,
            value_proc=DockerRunCreateConfig.check_is_valid_version,
        )
        registry_uri = click.prompt(
            click.style(
                "input registry uri, leave blank to set default as ->", fg="green"
            ),
            type=str,
            default="index.docker.io",
            show_default=True,
        )
        need_login = click.confirm(
            click.style("need login?", fg="green"), default=False
        )
        if need_login:
            click.echo(
                click.style(
                    "Leave it blank if you want use environment variables credentials, \n export DOCKER_USERNAME=xxx \n export DOCKER_PASSWORD=xxx",
                    fg="yellow",
                )
            )
            username = click.prompt(
                click.style("input username", fg="green"), default=""
            )
            password = click.prompt(
                click.style("input password", fg="green"), default=""
            )
            if username != "":
                self.config.set("registry", "docker_username", username)
            if password != "":
                self.config.set("registry", "docker_password", password)
        self.config.set("registry", "login", str(need_login))
        self.config.set("registry", "name", registry_uri)
        self.config.set("image", "name", image_name)
        self.config.set("image", "tag", tag)
        with open(self.args.filename, "w") as configfile:
            self.config.write(configfile)
